[PATCH v2] unpack-trees: fix accidentally quadratic behavior
- From
David Turner <dturner@twopensource.com>
- Date
- Jan 21, 2016, 21:11 UTC
- Message-ID
- <1453410708-23951-1-git-send-email-dturner@twopensource.com>
While unpacking trees (e.g. during git checkout), when we hit a cache entry that's past and outside our path, we cut off iteration.
This provides about a 45% speedup on git checkout between master and master^20000 on Twitter's monorepo. Speedup in general will depend on repostitory structure, number of changes, and packfile packing decisions.
Signed-off-by: David Turner <dturner@twopensource.com> --- unpack-trees.c | 13 ++++++++++++- 1 file changed, 12 insertions(+), 1 deletion(-)
diff --git a/unpack-trees.c b/unpack-trees.c index 5f541c2..d8e9685 100644 --- a/unpack-trees.c +++ b/unpack-trees.c @@ -695,8 +695,19 @@ static int find_cache_pos(struct traverse_info *info, ++o->cache_bottom; continue; } - if (!ce_in_traverse_path(ce, info)) + if (!ce_in_traverse_path(ce, info)) { + /* + * Check if we can skip future cache checks + * (because we're already past all possible + * entries in the traverse path). + */ + if (info->prev && info->traverse_path) { + if (strncmp(ce->name, info->traverse_path, + info->pathlen) > 0) + break; + } continue; + } ce_name = ce->name + pfxlen; ce_slash = strchr(ce_name, '/'); if (ce_slash)
-- 2.4.2.749.g730654d-twtrsrc